'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _KeyEventBitmapManager = require('../KeyEventBitmapManager');

var _KeyEventBitmapManager2 = _interopRequireDefault(_KeyEventBitmapManager);

var _KeyEventBitmapIndex = require('../../const/KeyEventBitmapIndex');

var _KeyEventBitmapIndex2 = _interopRequireDefault(_KeyEventBitmapIndex);

var _Logger = require('../Logger');

var _Logger2 = _interopRequireDefault(_Logger);

var _KeyCombinationSerializer = require('../KeyCombinationSerializer');

var _KeyCombinationSerializer2 = _interopRequireDefault(_KeyCombinationSerializer);

var _arrayFrom = require('../../utils/array/arrayFrom');

var _arrayFrom2 = _interopRequireDefault(_arrayFrom);

var _indexFromEnd = require('../../utils/array/indexFromEnd');

var _indexFromEnd2 = _interopRequireDefault(_indexFromEnd);

var _isObject = require('../../utils/object/isObject');

var _isObject2 = _interopRequireDefault(_isObject);

var _isUndefined = require('../../utils/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _isEmpty = require('../../utils/collection/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _describeKeyEvent = require('../../helpers/logging/describeKeyEvent');

var _describeKeyEvent2 = _interopRequireDefault(_describeKeyEvent);

var _KeyEventSequenceIndex = require('../../const/KeyEventSequenceIndex');

var _KeyEventSequenceIndex2 = _interopRequireDefault(_KeyEventSequenceIndex);

var _KeySequenceParser = require('../KeySequenceParser');

var _KeySequenceParser2 = _interopRequireDefault(_KeySequenceParser);

var _printComponent = require('../../helpers/logging/printComponent');

var _printComponent2 = _interopRequireDefault(_printComponent);

var _resolveUnaltShiftedAlias = require('../../helpers/resolving-handlers/resolveUnaltShiftedAlias');

var _resolveUnaltShiftedAlias2 = _interopRequireDefault(_resolveUnaltShiftedAlias);

var _resolveUnshiftedAlias = require('../../helpers/resolving-handlers/resolveUnshiftedAlias');

var _resolveUnshiftedAlias2 = _interopRequireDefault(_resolveUnshiftedAlias);

var _resolveUnaltedAlias = require('../../helpers/resolving-handlers/resolveUnaltedAlias');

var _resolveUnaltedAlias2 = _interopRequireDefault(_resolveUnaltedAlias);

var _resolveKeyAlias = require('../../helpers/resolving-handlers/resolveKeyAlias');

var _resolveKeyAlias2 = _interopRequireDefault(_resolveKeyAlias);

var _resolveAltShiftedAlias = require('../../helpers/resolving-handlers/resolveAltShiftedAlias');

var _resolveAltShiftedAlias2 = _interopRequireDefault(_resolveAltShiftedAlias);

var _resolveShiftedAlias = require('../../helpers/resolving-handlers/resolveShiftedAlias');

var _resolveShiftedAlias2 = _interopRequireDefault(_resolveShiftedAlias);

var _resolveAltedAlias = require('../../helpers/resolving-handlers/resolveAltedAlias');

var _resolveAltedAlias2 = _interopRequireDefault(_resolveAltedAlias);

var _Configuration = require('../Configuration');

var _Configuration2 = _interopRequireDefault(_Configuration);

var _ModifierFlagsDictionary = require('../../const/ModifierFlagsDictionary');

var _ModifierFlagsDictionary2 = _interopRequireDefault(_ModifierFlagsDictionary);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Defines common behaviour for key event strategies
 * @abstract
 * @class
 */
var AbstractKeyEventStrategy = function () {
  /********************************************************************************
   * Init & Reset
   ********************************************************************************/

  /**
   * Creates a new instance of a event strategy (this class is an abstract one and
   * not intended to be instantiated directly)
   * @param {Object} options Options for how event strategy should behave
   * @param {Logger} options.logger The Logger to use to report event strategy actions
   */
  function AbstractKeyEventStrategy() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var keyEventManager = arguments[1];

    _classCallCheck(this, AbstractKeyEventStrategy);

    this.logger = options.logger || new _Logger2.default('warn');
    /**
     * @typedef {Number} ComponentId Unique index associated with every HotKeys component
     * as it becomes active.
     *
     * For focus-only components, this happens when the component is focused. The HotKeys
     * component closest to the DOM element in focus gets the smallest number (0) and
     * those further up the render tree get larger (incrementing) numbers. When a different
     * element is focused (triggering the creation of a new focus tree) all component indexes
     * are reset (de-allocated) and re-assigned to the new tree of HotKeys components that
     * are now in focus.
     *
     * For global components, component indexes are assigned when a HotKeys component is
     * mounted, and de-allocated when it unmounts. The component index counter is never reset
     * back to 0 and just keeps incrementing as new components are mounted.
     */

    /**
     * Counter to maintain what the next component index should be
     * @type {ComponentId}
     */
    this.componentId = 0;

    /**
     * Reference to key event manager, so that information may pass between the
     * global strategy and the focus-only strategy
     * @type {KeyEventManager}
     */
    this.keyEventManager = keyEventManager;

    this._reset();
    this._resetKeyCombinationHistory();
  }

  /**
   * Resets all strategy state to the values it had when it was first created
   * @private
   */


  _createClass(AbstractKeyEventStrategy, [{
    key: '_reset',
    value: function _reset() {
      this._resetRegisteredKeyMapsState();
      this._resetHandlerResolutionState();
    }

    /**
     * Resets all state used to record information about the keymaps that HotKey
     * components have registered.
     *
     * After initialization, this state is generally maintained manually by
     * the _buildKeyMatcherMap() method and this method should not be called.
     */

  }, {
    key: '_resetRegisteredKeyMapsState',
    value: function _resetRegisteredKeyMapsState() {
      /**
       * Object containing a component's defined key maps and handlers
       * @typedef {Object} ComponentOptions
       * @property {ActionDictionary} actions - Dictionary of actions the component
       *          has defined in its keymap
       * @property {HandlersMap} handlers - Dictionary of handler functions the
       *          component has defined
       * @property {ComponentId} componentId - Index of the component the options
       *          correspond with
       */

      /**
       * List of actions and handlers registered by each component currently in focus.
       * The component closest to the element in focus is last in the list.
       * @type {ComponentOptions[]}
       */
      this.componentList = [];

      /**
       * Counter for the longest sequence registered by the HotKeys components currently
       * in focus. Allows setting an upper bound on the length of the key event history
       * that must be kept.
       * @type {Number}
       */
      this.longestSequence = 1;

      /**
       * The component index of the component that defines the longest key sequence, so
       * we can quickly determine if the longest sequence needs to be re-calculated when
       * that component is updated or removed.
       * @type {ComponentId}
       */
      this.longestSequenceComponentIndex = null;

      /**
       * Bitmap to record whether there is at least one keymap bound to each event type
       * (keydown, keypress or keyup) so that we can skip trying to find a matching keymap
       * on events where we know there is none to find
       * @type {KeyEventBitmap}
       */
      this.keyMapEventBitmap = _KeyEventBitmapManager2.default.newBitmap();
    }

    /**
     * Resets the state of the values used to resolve which handler function should be
     * called when key events match a registered key map
     * @private
     */

  }, {
    key: '_resetHandlerResolutionState',
    value: function _resetHandlerResolutionState() {
      /**
       * List of mappings from key sequences to handlers that is constructed on-the-fly
       * as key events propagate up the render tree
       */
      this.keyMaps = null;

      /**
       * Index marking the number of places from the end of componentList for which the
       * keyMaps have been matched with event handlers. Used to build this.keyMaps as
       * key events propagate up the React tree.
       * @type {Number}
       */
      this.handlerResolutionSearchIndex = 0;

      /**
       * Array of counters - one for each component - to keep track of how many handlers
       * for that component still need actions assigned to them
       * @type {Number[]}
       */
      this.unmatchedHandlerStatus = null;

      /**
       * A dictionary of handlers to the components that register them. This is populated
       * as this.handlerResolutionSearchIndex increases, moving from the end of this.componentList to the
       * front, populating this.keyMaps as needed
       * @type {Object<ActionName, ComponentId>}
       */
      this.handlersDictionary = {};

      /**
       * A dictionary of sequences already encountered in the process of building the
       * list of keyMaps on the fly, as key events propagate up the component tree
       */
      this.keySequencesDictionary = {};
    }

    /**
     * Reset the state values that record the current and recent state of key events
     * @private
     */

  }, {
    key: '_resetKeyCombinationHistory',
    value: function _resetKeyCombinationHistory() {
      /**
       * Whether the current key combination includes at least one keyup event - indicating
       * that the current combination is ending (and keys are being released)
       */
      this.keyCombinationIncludesKeyUp = false;

      this.keypressEventsToSimulate = [];

      if (!this.keyCombinationHistory || this.keyCombinationHistory.length < 1) {
        this.keyCombinationHistory = [];
      } else {
        var currentKeyCombination = this._getCurrentKeyCombination();

        var keysStillPressed = Object.keys(currentKeyCombination.keys).reduce(function (memo, keyName) {
          var keyState = currentKeyCombination.keys[keyName];
          var currentKeyState = keyState[_KeyEventSequenceIndex2.default.current];

          if (currentKeyState[_KeyEventBitmapIndex2.default.keydown] && !currentKeyState[_KeyEventBitmapIndex2.default.keyup]) {
            memo[keyName] = keyState;
          }

          return memo;
        }, {});

        this.keyCombinationHistory = [{
          keys: keysStillPressed,
          ids: _KeyCombinationSerializer2.default.serialize(keysStillPressed)
        }];
      }
    }

    /********************************************************************************
     * Registering key maps and handlers
     ********************************************************************************/

    /**
     * Registers the hotkeys defined by a HotKeys component
     * @param {ComponentId} componentId - Index of the component
     * @param {KeyMap} actionNameToKeyMap - Definition of actions and key maps defined
     *        in the HotKeys component
     * @param {HandlersMap} actionNameToHandlersMap - Map of ActionNames to handlers
     *        defined in the HotKeys component
     * @param {Object} options - Hash of options that configure how the key map is built.
     * @private
     */

  }, {
    key: '_addComponentToList',
    value: function _addComponentToList(componentId) {
      var actionNameToKeyMap = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var actionNameToHandlersMap = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var options = arguments[3];

      var componentOptions = this._buildComponentOptions(componentId, actionNameToKeyMap, actionNameToHandlersMap, options);

      this.componentList.push(componentOptions);

      return componentOptions;
    }

    /**
     * Builds the internal representation that described the options passed to a HotKeys
     * component
     * @param {ComponentId} componentId - Index of the component
     * @param {KeyMap} actionNameToKeyMap - Definition of actions and key maps defined
     *        in the HotKeys component
     * @param {HandlersMap} actionNameToHandlersMap - Map of ActionNames to handlers
     *        defined in the HotKeys component
     * @param {Object} options - Hash of options that configure how the key map is built.
     * @param {String} options.defaultKeyEvent - The default key event to use for any
     *        action that does not explicitly define one.
     * @returns {ComponentOptions} Options for the specified component
     * @private
     */

  }, {
    key: '_buildComponentOptions',
    value: function _buildComponentOptions(componentId, actionNameToKeyMap, actionNameToHandlersMap, options) {
      var _applyHardSequences2 = this._applyHardSequences(actionNameToKeyMap, actionNameToHandlersMap),
          hardSequenceKeyMap = _applyHardSequences2.keyMap,
          includingHardSequenceHandlers = _applyHardSequences2.handlers;

      return {
        actions: this._buildActionDictionary(_extends({}, actionNameToKeyMap, hardSequenceKeyMap), options, componentId),
        handlers: includingHardSequenceHandlers,
        componentId: componentId,
        options: options
      };
    }

    /**
     * Applies hard sequences (handlers attached to actions with names that are valid
     * KeySequenceStrings) that implicitly define a corresponding action name.
     * @param {KeyMap} actionNameToKeyMap - KeyMap specified by HotKeys component
     * @param {HandlersMap} actionNameToHandlersMap - HandlersMap specified by HotKeys
     *        component
     * @returns {{keyMap: {}, handlers: {}}} Object containing keymap and handlers map
     *        with the hard sequence actions applied
     * @private
     */

  }, {
    key: '_applyHardSequences',
    value: function _applyHardSequences(actionNameToKeyMap, actionNameToHandlersMap) {
      if (_Configuration2.default.option('enableHardSequences')) {
        return Object.keys(actionNameToHandlersMap).reduce(function (memo, actionNameOrKeyExpression) {
          var actionNameIsInKeyMap = !!actionNameToKeyMap[actionNameOrKeyExpression];

          if (!actionNameIsInKeyMap && _KeyCombinationSerializer2.default.isValidKeySerialization(actionNameOrKeyExpression)) {
            memo.keyMap[actionNameOrKeyExpression] = actionNameOrKeyExpression;
          }

          memo.handlers[actionNameOrKeyExpression] = actionNameToHandlersMap[actionNameOrKeyExpression];

          return memo;
        }, { keyMap: {}, handlers: {} });
      } else {
        return { keyMap: actionNameToKeyMap, handlers: actionNameToHandlersMap };
      }
    }

    /**
     * Object containing all the information required to match a key event to an action
     * @typedef {Object} ActionConfiguration
     * @property {KeyCombinationString} id - String description of keys involved in the
     *          final key combination in the sequence
     * @property {ActionName} actionName - Name of the action associated with the key map
     * @property {NormalizedKeySequenceId} prefix - String describing sequence of key
     *          combinations involved key map, before the final key combination
     * @property {Number} sequenceLength - Number of combinations involved in the
     *           sequence
     * @property {Number} size - Number of keys involved in the combination
     * @property {Object.<KeyName, Boolean>} keyDictionary - Dictionary of key names involved
     *           in the key combination
     * @property {KeyEventBitmapIndex} eventBitmapIndex - Bitmap index for key event that
     *          the matcher should match on
     */

    /**
     * A mapping between ActionNames and FullKeyEventOptions
     * @typedef {Object<ActionName,ActionConfiguration>} ActionDictionary
     */

    /**
     * Returns a mapping between ActionNames and FullKeyEventOptions
     * @param {KeyMap} actionNameToKeyMap - Mapping of ActionNames to key sequences.
     * @param {Object} options - Hash of options that configure how the key map is built.
     * @param {String} options.defaultKeyEvent - The default key event to use for any
     *        action that does not explicitly define one.
     * @param {ComponentId} componentId Index of the component the matcher belongs to
     * @return {ActionDictionary} Map from ActionNames to FullKeyEventOptions
     * @private
     */

  }, {
    key: '_buildActionDictionary',
    value: function _buildActionDictionary(actionNameToKeyMap, options, componentId) {
      var _this = this;

      return Object.keys(actionNameToKeyMap).reduce(function (keyMapMemo, actionName) {
        var keyMapOptions = (0, _arrayFrom2.default)(actionNameToKeyMap[actionName]);

        keyMapOptions.forEach(function (keyMapOption) {
          var _ref = function () {
            if ((0, _isObject2.default)(keyMapOption)) {
              var _sequence = keyMapOption.sequence,
                  action = keyMapOption.action;


              return {
                keySequence: _sequence,
                eventBitmapIndex: (0, _isUndefined2.default)(action) ? _KeyEventBitmapIndex2.default[options.defaultKeyEvent] : _KeyEventBitmapIndex2.default[action]
              };
            } else {
              return {
                keySequence: keyMapOption,
                eventBitmapIndex: _KeyEventBitmapIndex2.default[options.defaultKeyEvent]
              };
            }
          }(),
              keySequence = _ref.keySequence,
              eventBitmapIndex = _ref.eventBitmapIndex;

          var _KeySequenceParser$pa = _KeySequenceParser2.default.parse(keySequence, { eventBitmapIndex: eventBitmapIndex }),
              sequence = _KeySequenceParser$pa.sequence,
              combination = _KeySequenceParser$pa.combination;

          if (sequence.size > _this.longestSequence) {
            _this.longestSequence = sequence.size;
            _this.longestSequenceComponentIndex = componentId;
          }

          /**
           * Record that there is at least one key sequence in the focus tree bound to
           * the keyboard event
           */
          _KeyEventBitmapManager2.default.setBit(_this.keyMapEventBitmap, eventBitmapIndex);

          if (!keyMapMemo[actionName]) {
            keyMapMemo[actionName] = [];
          }

          keyMapMemo[actionName].push(_extends({
            prefix: sequence.prefix,
            actionName: actionName,
            sequenceLength: sequence.size
          }, combination));
        });

        return keyMapMemo;
      }, {});
    }

    /********************************************************************************
     * Recording key events
     ********************************************************************************/

    /**
     * Record of the combination of keys that are currently being pressed
     * @typedef {Object} KeyCombinationRecord
     * @param {Object<ReactKeyName, KeyEventBitmap[]>} keys - A dictionary
     * of keys that have been pressed down at once. The keys of the map are the lowercase
     * names of the keyboard keys. May contain 1 or more keyboard keys.
     * @param {KeySequenceString} ids - Serialization of keys currently pressed in
     *        combination
     */

    /**
     * Returns the current key combination, i.e. the key combination that represents
     * the current key events.
     * @returns {KeyCombinationRecord} The current key combination
     * @private
     */

  }, {
    key: '_getCurrentKeyCombination',
    value: function _getCurrentKeyCombination() {
      if (this.keyCombinationHistory.length > 0) {
        return this.keyCombinationHistory[this.keyCombinationHistory.length - 1];
      } else {
        return { keys: {}, ids: [''], keyAliases: {} };
      }
    }

    /**
     * Adds a key event to the current key combination (as opposed to starting a new
     * keyboard combination).
     * @param {ReactKeyName} keyName - Name of the key to add to the current combination
     * @param {KeyEventBitmapIndex} bitmapIndex - Index in bitmap to set to true
     * @private
     */

  }, {
    key: '_addToCurrentKeyCombination',
    value: function _addToCurrentKeyCombination(keyName, bitmapIndex) {
      if (this.keyCombinationHistory.length === 0) {
        this.keyCombinationHistory.push({ keys: {}, ids: [''], keyAliases: {} });
      }

      var keyCombination = this._getCurrentKeyCombination();
      var keyAlias = this._getKeyAlias(keyCombination, keyName);

      var existingBitmap = this._getKeyState(keyCombination, keyName);

      if (!existingBitmap) {
        keyCombination.keys[keyAlias] = [_KeyEventBitmapManager2.default.newBitmap(), _KeyEventBitmapManager2.default.newBitmap(bitmapIndex)];
      } else {
        keyCombination.keys[keyAlias] = [_KeyEventBitmapManager2.default.clone(existingBitmap[1]), _KeyEventBitmapManager2.default.newBitmap(bitmapIndex)];
      }

      keyCombination.ids = _KeyCombinationSerializer2.default.serialize(keyCombination.keys);
      keyCombination.keyAliases = this._buildCombinationKeyAliases(keyCombination.keys);

      if (bitmapIndex === _KeyEventBitmapIndex2.default.keyup) {
        this.keyCombinationIncludesKeyUp = true;
      }
    }

    /**
     * Adds a new KeyCombinationRecord to the event history and resets the keystateIncludesKeyUp
     * flag to false.
     * @param {ReactKeyName} keyName - Name of the keyboard key to add to the new
     *        KeyCombinationRecord
     * @param {KeyEventBitmapIndex} eventBitmapIndex - Index of bit to set to true in new
     *        KeyEventBitmap
     * @private
     */

  }, {
    key: '_startNewKeyCombination',
    value: function _startNewKeyCombination(keyName, eventBitmapIndex) {
      if (this.keyCombinationHistory.length > this.longestSequence) {
        /**
         * We know the longest key sequence registered for the currently focused
         * components, so we don't need to keep a record of history longer than
         * that
         */
        this.keyCombinationHistory.shift();
      }

      var lastKeyCombination = this._getCurrentKeyCombination();

      var keys = _extends({}, this._withoutKeyUps(lastKeyCombination), _defineProperty({}, keyName, [_KeyEventBitmapManager2.default.newBitmap(), _KeyEventBitmapManager2.default.newBitmap(eventBitmapIndex)]));

      this.keyCombinationHistory.push({
        keys: keys,
        ids: _KeyCombinationSerializer2.default.serialize(keys),
        keyAliases: this._buildCombinationKeyAliases(keys)
      });

      this.keyCombinationIncludesKeyUp = false;
    }

    /**
     * Returns a new KeyCombinationRecord without the keys that have been
     * released (had the keyup event recorded). Essentially, the keys that are
     * currently still pressed down at the time a key event is being handled.
     * @param {KeyCombinationRecord} keyCombinationRecord Record of keys currently
     *        pressed down that should have the release keyed omitted from
     * @returns {KeyCombinationRecord} New KeyCombinationRecord with all of the
     *        keys with keyup events omitted
     * @private
     */

  }, {
    key: '_withoutKeyUps',
    value: function _withoutKeyUps(keyCombinationRecord) {
      return Object.keys(keyCombinationRecord.keys).reduce(function (memo, keyName) {
        var keyState = keyCombinationRecord.keys[keyName];

        if (!keyState[_KeyEventSequenceIndex2.default.current][_KeyEventBitmapIndex2.default.keyup]) {
          memo[keyName] = keyState;
        }

        return memo;
      }, {});
    }

    /********************************************************************************
     * Matching and calling handlers
     ********************************************************************************/

  }, {
    key: '_callMatchingHandlerClosestToEventTarget',
    value: function _callMatchingHandlerClosestToEventTarget(event, keyName, eventBitmapIndex, componentId, componentSearchIndex) {
      var _this2 = this;

      if (!this.keyMaps || !this.unmatchedHandlerStatus) {
        this.keyMaps = [];

        this.unmatchedHandlerStatus = [];

        this.componentList.forEach(function (_ref2) {
          var handlers = _ref2.handlers;

          _this2.unmatchedHandlerStatus.push([Object.keys(handlers).length, {}]);
          _this2.keyMaps.push({});
        });
      }

      while (componentSearchIndex <= componentId) {
        var unmatchedHandlersStatus = this.unmatchedHandlerStatus[componentSearchIndex];
        var unmatchedHandlersCount = unmatchedHandlersStatus[0];

        if (unmatchedHandlersCount > 0) {
          var _loop = function _loop() {
            var _componentList$handle = _this2.componentList[_this2.handlerResolutionSearchIndex],
                handlers = _componentList$handle.handlers,
                actions = _componentList$handle.actions;

            /**
             * Add current component's handlers to the handlersDictionary so we know
             * which component has defined them
             */

            Object.keys(handlers).forEach(function (actionName) {
              if (!_this2.handlersDictionary[actionName]) {
                _this2.handlersDictionary[actionName] = [];
              }

              _this2.handlersDictionary[actionName].push(_this2.handlerResolutionSearchIndex);
            });

            /**
             * Iterate over the actions of a component (starting with the current component
             * and working through its ancestors), matching them to the current component's
             * handlers
             */
            Object.keys(actions).forEach(function (actionName) {
              var handlerComponentIndexArray = _this2.handlersDictionary[actionName];

              if (handlerComponentIndexArray) {
                /**
                 * Get action handler closest to the event target
                 */
                var handlerComponentIndex = handlerComponentIndexArray[0];

                var handler = _this2.componentList[handlerComponentIndex].handlers[actionName];

                /**
                 * Get key map that corresponds with the component that defines the handler
                 * closest to the event target
                 */
                var _keyMap = _this2.keyMaps[handlerComponentIndex];

                /**
                 * Store the key sequence with the handler that it should call at
                 * a given component level
                 */
                if (!_keyMap.sequences) {
                  _keyMap.sequences = {};
                }

                /**
                 * At least one child HotKeys component (or the component itself) has
                 * defined a handler for the action, so now we need to associate them
                 */
                var keyMatchers = actions[actionName];

                keyMatchers.forEach(function (keyMatcher) {
                  var keySequence = [keyMatcher.prefix, keyMatcher.id].join(' ');

                  var closestSequenceHandlerAlreadyFound = _this2.keySequencesDictionary[keySequence] && _this2.keySequencesDictionary[keySequence].some(function (dictEntry) {
                    return dictEntry[1] === keyMatcher.eventBitmapIndex;
                  });

                  if (closestSequenceHandlerAlreadyFound) {
                    /**
                     * Return if there is already a component with handlers for the current
                     * key sequence closer to the event target
                     */
                    return;
                  }

                  if (!_keyMap.sequences[keyMatcher.prefix]) {
                    _keyMap.sequences[keyMatcher.prefix] = { combinations: {} };
                  }

                  var prefix = keyMatcher.prefix,
                      sequenceLength = keyMatcher.sequenceLength,
                      id = keyMatcher.id,
                      keyDictionary = keyMatcher.keyDictionary,
                      size = keyMatcher.size,
                      matcherEventBitmapIndex = keyMatcher.eventBitmapIndex,
                      actionName = keyMatcher.actionName;


                  var combination = _keyMap.sequences[keyMatcher.prefix].combinations[keyMatcher.id];

                  if (!combination) {
                    _keyMap.sequences[keyMatcher.prefix].combinations[keyMatcher.id] = {
                      prefix: prefix, sequenceLength: sequenceLength, id: id, keyDictionary: keyDictionary, size: size,
                      events: _defineProperty({}, matcherEventBitmapIndex, {
                        actionName: actionName, eventBitmapIndex: matcherEventBitmapIndex, handler: handler
                      })
                    };
                  } else {
                    _keyMap.sequences[keyMatcher.prefix].combinations[keyMatcher.id] = _extends({}, combination, {
                      events: _extends({}, combination.events, _defineProperty({}, matcherEventBitmapIndex, {
                        actionName: actionName, eventBitmapIndex: matcherEventBitmapIndex, handler: handler
                      }))
                    });
                  }

                  /**
                   * Merge event bitmaps so we can quickly determine if a given component
                   * has any handlers bound to particular key events
                   */
                  if (!_keyMap.eventBitmap) {
                    _keyMap.eventBitmap = _KeyEventBitmapManager2.default.newBitmap();
                  }

                  _KeyEventBitmapManager2.default.setBit(_keyMap.eventBitmap, keyMatcher.eventBitmapIndex);

                  /**
                   * Record the longest sequence length so we know to only check for sequences
                   * of that length or shorter for a particular component
                   */
                  if (!_keyMap.longestSequence || _keyMap.longestSequence < keyMatcher.sequenceLength) {
                    _keyMap.longestSequence = keyMatcher.sequenceLength;
                  }

                  /**
                   * Record that we have already found a handler for the current action so
                   * that we do not override handlers for an action closest to the event target
                   * with handlers further up the tree
                   */
                  if (!_this2.keySequencesDictionary[keySequence]) {
                    _this2.keySequencesDictionary[keySequence] = [];
                  }

                  _this2.keySequencesDictionary[keySequence].push([handlerComponentIndex, keyMatcher.eventBitmapIndex]);
                });

                handlerComponentIndexArray.forEach(function (handlerComponentIndex) {
                  var handlerComponentStatus = _this2.unmatchedHandlerStatus[handlerComponentIndex];

                  if (!handlerComponentStatus[1][actionName]) {
                    handlerComponentStatus[1][actionName] = true;

                    /**
                     * Decrement the number of remaining unmatched handlers for the
                     * component currently handling the propagating key event, so we know
                     * when all handlers have been matched to sequences and we can move on
                     * to matching them against the current key event
                     */
                    handlerComponentStatus[0]--;
                  }
                });
              }
            });

            /**
             * Search next component up in the hierarchy for actions that match outstanding
             * handlers
             */
            _this2.handlerResolutionSearchIndex++;
          };

          /**
           * Component currently handling key event has handlers that have not yet been
           * associated with a key sequence. We need to continue walking up the component
           * tree in search of the matching actions that describe the applicable key
           * sequence.
           */

          while (this.handlerResolutionSearchIndex < this.componentList.length && unmatchedHandlersCount > 0) {
            _loop();
          }
        }

        var keyMap = this.keyMaps[componentSearchIndex];

        this.logger.verbose(this._logPrefix(componentSearchIndex) + ' Internal key mapping:\n', '' + (0, _printComponent2.default)(keyMap));

        if (!keyMap || (0, _isEmpty2.default)(keyMap.sequences) || !keyMap.eventBitmap[eventBitmapIndex]) {
          /**
           * Component doesn't define any matchers for the current key event
           */

          this.logger.debug(this._logPrefix(componentSearchIndex) + ' Doesn\'t define a handler for \'' + this._describeCurrentKeyCombination() + '\' ' + (0, _describeKeyEvent2.default)(eventBitmapIndex) + '.');
        } else {
          var sequences = keyMap.sequences,
              longestSequence = keyMap.longestSequence;


          var currentKeyState = this._getCurrentKeyCombination();
          var normalizedKeyName = this._getKeyAlias(currentKeyState, keyName);

          var sequenceLengthCounter = longestSequence;

          while (sequenceLengthCounter >= 0) {
            var sequenceHistory = this.keyCombinationHistory.slice(-sequenceLengthCounter, -1);
            var sequenceHistoryIds = sequenceHistory.map(function (_ref3) {
              var ids = _ref3.ids;
              return ids;
            });

            var matchingSequence = this._tryMatchSequenceWithKeyAliases(sequences, sequenceHistoryIds);

            if (matchingSequence) {
              if (!matchingSequence.order) {
                (function () {
                  /**
                   * The first time the component that is currently handling the key event has
                   * its handlers searched for a match, order the combinations based on their
                   * size so that they may be applied in the correct priority order
                   */

                  var combinationsPartitionedBySize = Object.values(matchingSequence.combinations).reduce(function (memo, _ref4) {
                    var id = _ref4.id,
                        size = _ref4.size;

                    if (!memo[size]) {
                      memo[size] = [];
                    }

                    memo[size].push(id);

                    return memo;
                  }, {});

                  matchingSequence.order = Object.keys(combinationsPartitionedBySize).sort(function (a, b) {
                    return b - a;
                  }).reduce(function (memo, key) {
                    return memo.concat(combinationsPartitionedBySize[key]);
                  }, []);
                })();
              }

              var combinationOrder = matchingSequence.order;

              var combinationIndex = 0;

              while (combinationIndex < combinationOrder.length) {
                var combinationId = combinationOrder[combinationIndex];
                var combinationMatcher = matchingSequence.combinations[combinationId];

                if (this._combinationMatchesKeys(normalizedKeyName, currentKeyState, combinationMatcher, eventBitmapIndex)) {
                  var subMatchDescription = _KeyCombinationSerializer2.default.serialize(combinationMatcher.keyDictionary);

                  this.logger.debug(this._logPrefix(componentSearchIndex) + ' Found action that matches \'' + this._describeCurrentKeyCombination() + '\' (sub-match: \'' + subMatchDescription + '\'): ' + combinationMatcher.events[eventBitmapIndex].actionName + '. Calling handler . . .');
                  combinationMatcher.events[eventBitmapIndex].handler(event);

                  this._stopEventPropagationAfterHandlingIfEnabled(event, componentSearchIndex);

                  return true;
                }

                combinationIndex++;
              }
            }

            sequenceLengthCounter--;
          }

          var eventName = (0, _describeKeyEvent2.default)(eventBitmapIndex);
          this.logger.debug(this._logPrefix(componentSearchIndex) + ' No matching actions found for \'' + this._describeCurrentKeyCombination() + '\' ' + eventName + '.');
        }

        componentSearchIndex++;
      }
    }
  }, {
    key: '_stopEventPropagationAfterHandlingIfEnabled',
    value: function _stopEventPropagationAfterHandlingIfEnabled(event, componentId) {
      if (_Configuration2.default.option('stopEventPropagationAfterHandling')) {
        this._stopEventPropagation(event, componentId);

        return true;
      }

      return false;
    }
  }, {
    key: '_stopEventPropagationAfterIgnoringIfEnabled',
    value: function _stopEventPropagationAfterIgnoringIfEnabled(event, componentId) {
      if (_Configuration2.default.option('stopEventPropagationAfterIgnoring')) {
        this._stopEventPropagation(event, componentId);

        return true;
      }

      return false;
    }
  }, {
    key: '_stopEventPropagation',
    value: function _stopEventPropagation(event, componentId) {
      this.logger.debug(this._logPrefix(componentId) + ' Stopping further event propagation.');

      event.stopPropagation();
    }
  }, {
    key: '_describeCurrentKeyCombination',
    value: function _describeCurrentKeyCombination() {
      return this._getCurrentKeyCombination().ids[0];
    }
  }, {
    key: '_tryMatchSequenceWithKeyAliases',
    value: function _tryMatchSequenceWithKeyAliases(keyMatcher, sequenceIds) {
      if (sequenceIds.length === 0) {
        return keyMatcher[''];
      }

      var idSizes = sequenceIds.map(function (ids) {
        return ids.length;
      });
      var indexCounters = sequenceIds.map(function () {
        return 0;
      });

      var triedAllPossiblePermutations = false;

      while (!triedAllPossiblePermutations) {
        var sequenceIdPermutation = indexCounters.map(function (sequenceIdIndex, index) {
          return sequenceIds[index][sequenceIdIndex];
        });

        var candidateId = sequenceIdPermutation.join(' ');

        if (keyMatcher[candidateId]) {
          return keyMatcher[candidateId];
        }

        var incrementer = 0;
        var carry = true;

        while (carry && incrementer < indexCounters.length) {
          var count = (0, _indexFromEnd2.default)(indexCounters, incrementer);

          var newIndex = (count + 1) % ((0, _indexFromEnd2.default)(idSizes, incrementer) || 1);

          indexCounters[indexCounters.length - (incrementer + 1)] = newIndex;

          carry = newIndex === 0;

          if (carry) {
            incrementer++;
          }
        }

        triedAllPossiblePermutations = incrementer === indexCounters.length;
      }
    }
  }, {
    key: '_combinationMatchesKeys',
    value: function _combinationMatchesKeys(keyBeingPressed, keyCombination, combinationMatch, eventBitmapIndex) {
      var _this3 = this;

      var combinationHasHandlerForEventType = combinationMatch.events[eventBitmapIndex];

      if (!combinationHasHandlerForEventType) {
        return false;
      }

      var keyCompletesCombination = false;

      var combinationMatchesKeysPressed = !Object.keys(combinationMatch.keyDictionary).some(function (candidateKeyName) {
        var keyState = _this3._getKeyState(keyCombination, candidateKeyName);

        if (keyState) {
          if (_this3._keyIsCurrentlyTriggeringEvent(keyState, eventBitmapIndex)) {
            if (keyBeingPressed && keyBeingPressed === _this3._getKeyAlias(keyCombination, candidateKeyName)) {
              keyCompletesCombination = !_this3._keyAlreadyTriggeredEvent(keyState, eventBitmapIndex);
            }

            return false;
          } else {
            return true;
          }
        } else {
          return true;
        }
      });

      return combinationMatchesKeysPressed && keyCompletesCombination;
    }

    /**
     * Synchronises the key combination history to match the modifier key flag attributes
     * on new key events
     * @param {KeyboardEvent} event - Event to check the modifier flags for
     * @private
     */

  }, {
    key: '_checkForModifierFlagDiscrepancies',
    value: function _checkForModifierFlagDiscrepancies(event) {
      var _this4 = this;

      /**
       * If a new key event is received with modifier key flags that contradict the
       * key combination history we are maintaining, we can surmise that some keyup events
       * for those modifier keys have been lost (possibly because the window lost focus).
       * We update the key combination to match the modifier flags
       */
      Object.keys(_ModifierFlagsDictionary2.default).forEach(function (modifierKey) {
        var modifierKeyState = _this4._getCurrentKeyState(modifierKey);
        var modifierStillPressed = modifierKeyState && !_this4._keyIsCurrentlyTriggeringEvent(modifierKeyState, _KeyEventBitmapIndex2.default.keyup);

        _ModifierFlagsDictionary2.default[modifierKey].forEach(function (attributeName) {
          if (event[attributeName] === false && modifierStillPressed) {
            _this4._addToCurrentKeyCombination(modifierKey, _KeyEventBitmapIndex2.default.keyup);
          }
        });
      });
    }
  }, {
    key: '_keyIsCurrentlyTriggeringEvent',
    value: function _keyIsCurrentlyTriggeringEvent(keyState, eventBitmapIndex) {
      return keyState && keyState[_KeyEventSequenceIndex2.default.current][eventBitmapIndex];
    }
  }, {
    key: '_keyAlreadyTriggeredEvent',
    value: function _keyAlreadyTriggeredEvent(keyState, eventBitmapIndex) {
      return keyState && keyState[_KeyEventSequenceIndex2.default.previous][eventBitmapIndex];
    }
  }, {
    key: '_getCurrentKeyState',
    value: function _getCurrentKeyState(keyName) {
      var currentCombination = this._getCurrentKeyCombination();

      return this._getKeyState(currentCombination, keyName);
    }
  }, {
    key: '_getKeyState',
    value: function _getKeyState(keyCombination, keyName) {
      var keyState = keyCombination.keys[keyName];

      if (keyState) {
        return keyState;
      } else {
        var keyAlias = keyCombination.keyAliases[keyName];

        if (keyAlias) {
          return keyCombination.keys[keyAlias];
        }
      }
    }
  }, {
    key: '_getKeyAlias',
    value: function _getKeyAlias(keyCombination, keyName) {
      var keyState = keyCombination.keys[keyName];

      if (keyState) {
        return keyName;
      } else {
        var keyAlias = keyCombination.keyAliases[keyName];

        if (keyAlias) {
          return keyAlias;
        } else {
          return keyName;
        }
      }
    }
  }, {
    key: '_buildCombinationKeyAliases',
    value: function _buildCombinationKeyAliases(keyDictionary) {
      var aliasFunctions = function () {
        if (keyDictionary['Shift']) {
          if (keyDictionary['Alt']) {
            return [_resolveAltShiftedAlias2.default, _resolveUnaltShiftedAlias2.default];
          } else {
            return [_resolveShiftedAlias2.default, _resolveUnshiftedAlias2.default];
          }
        } else {
          if (keyDictionary['Alt']) {
            return [_resolveAltedAlias2.default, _resolveUnaltedAlias2.default];
          } else {
            var nop = function nop(keyName) {
              return [keyName];
            };
            return [nop, nop];
          }
        }
      }();

      return Object.keys(keyDictionary).reduce(function (memo, keyName) {
        (0, _resolveKeyAlias2.default)(keyName).forEach(function (normalizedKey) {
          aliasFunctions.forEach(function (aliasFunction) {
            aliasFunction(normalizedKey).forEach(function (keyAlias) {
              if (keyAlias !== keyName || keyName !== normalizedKey) {
                memo[keyAlias] = keyName;
              }
            });
          });
        });

        return memo;
      }, {});
    }
  }]);

  return AbstractKeyEventStrategy;
}();

exports.default = AbstractKeyEventStrategy;