'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _AbstractKeyEventStrategy = require('./AbstractKeyEventStrategy');

var _AbstractKeyEventStrategy2 = _interopRequireDefault(_AbstractKeyEventStrategy);

var _KeyEventBitmapIndex = require('../../const/KeyEventBitmapIndex');

var _KeyEventBitmapIndex2 = _interopRequireDefault(_KeyEventBitmapIndex);

var _KeyEventSequenceIndex = require('../../const/KeyEventSequenceIndex');

var _KeyEventSequenceIndex2 = _interopRequireDefault(_KeyEventSequenceIndex);

var _KeyEventCounter = require('../KeyEventCounter');

var _KeyEventCounter2 = _interopRequireDefault(_KeyEventCounter);

var _hasKeyPressEvent = require('../../helpers/resolving-handlers/hasKeyPressEvent');

var _hasKeyPressEvent2 = _interopRequireDefault(_hasKeyPressEvent);

var _describeKeyEvent = require('../../helpers/logging/describeKeyEvent');

var _describeKeyEvent2 = _interopRequireDefault(_describeKeyEvent);

var _Configuration = require('../Configuration');

var _Configuration2 = _interopRequireDefault(_Configuration);

var _Logger = require('../Logger');

var _Logger2 = _interopRequireDefault(_Logger);

var _printComponent = require('../../helpers/logging/printComponent');

var _printComponent2 = _interopRequireDefault(_printComponent);

var _isUndefined = require('../../utils/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _normalizeKeyName = require('../../helpers/resolving-handlers/normalizeKeyName');

var _normalizeKeyName2 = _interopRequireDefault(_normalizeKeyName);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Defines behaviour for dealing with key maps defined in focus-only HotKey components
 * @class
 */
var FocusOnlyKeyEventStrategy = function (_AbstractKeyEventStra) {
  _inherits(FocusOnlyKeyEventStrategy, _AbstractKeyEventStra);

  /********************************************************************************
   * Init & Reset
   ********************************************************************************/

  function FocusOnlyKeyEventStrategy() {
    var configuration = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var keyEventManager = arguments[1];

    _classCallCheck(this, FocusOnlyKeyEventStrategy);

    /**
     * State that doesn't get cleared on each new focus tree
     */

    /**
     * Unique identifier given to each focus tree - when the focus in the browser
     * changes, and a different tree of elements are focused, a new id is allocated
     * @typedef {Number} FocusTreeId
     */

    /**
     * Counter to keep track of what focus tree ID should be allocated next
     * @type {FocusTreeId}
     */
    var _this = _possibleConstructorReturn(this, (FocusOnlyKeyEventStrategy.__proto__ || Object.getPrototypeOf(FocusOnlyKeyEventStrategy)).call(this, configuration, keyEventManager));
    /**
     * Set state that DOES get cleared on each new focus tree
     */


    _this.focusTreeId = 0;

    /**
     * Record of the event currently bubbling up through the React application (and
     * beyond). This state is *not* cleared when the event propagation is finished
     * or when the component focus tree changes. It persists until it is overridden
     * by a new event, so that the global strategy is able to inspect the last
     * event seen by the React application, even after focus is lost.
     */
    _this.currentEvent = {
      /**
       * The name of the key the event belongs to
       * @type {ReactKeyName}
       */
      key: null,

      /**
       * The event bitmap index of the type of key event
       * @type {KeyEventBitmapIndex}
       */
      type: null,

      handled: false
    };
    return _this;
  }

  /**
   * Clears the internal state, wiping any history of key events and registered handlers
   * so they have no effect on the next tree of focused HotKeys components
   * @private
   */


  _createClass(FocusOnlyKeyEventStrategy, [{
    key: '_reset',
    value: function _reset() {
      _get(FocusOnlyKeyEventStrategy.prototype.__proto__ || Object.getPrototypeOf(FocusOnlyKeyEventStrategy.prototype), '_reset', this).call(this);

      /**
       * Increase the unique ID associated with each unique focus tree
       * @type {number}
       */
      this.componentId = 0;

      this.focusTreeId += 1;

      this._clearEventPropagationState();
    }

    /**
     * Clears the history that is maintained for the duration of a single keyboard event's
     * propagation up the React component tree towards the root component, so that the
     * next keyboard event starts with a clean state.
     * @private
     */

  }, {
    key: '_clearEventPropagationState',
    value: function _clearEventPropagationState() {
      /**
       * Object containing state of a key events propagation up the render tree towards
       * the document root
       * @type {{previousComponentIndex: number, actionHandled: boolean}}}
       */
      this.eventPropagationState = {
        /**
         * Index of the component last seen to be handling a key event
         * @type {ComponentId}
         */
        previousComponentIndex: -1,

        /**
         * Whether the keyboard event currently being handled has already matched a
         * handler function that has been called
         * @type {Boolean}
         */
        actionHandled: false,

        /**
         * Whether the keyboard event current being handled should be ignored
         * @type {Boolean}
         */
        ignoreEvent: false
      };
    }

    /********************************************************************************
     * Registering key maps and handlers
     ********************************************************************************/

    /**
     * Registers the actions and handlers of a HotKeys component that has gained focus
     * @param {KeyMap} actionNameToKeyMap - Map of actions to key expressions
     * @param {HandlersMap} actionNameToHandlersMap - Map of actions to handler functions
     * @param {Object} options Hash of options that configure how the actions
     *        and handlers are associated and called.
     * @returns {[FocusTreeId, ComponentId]} The current focus tree's ID and a unique
     *         component ID to assign to the focused HotKeys component and passed back
     *         when handling a key event
     */

  }, {
    key: 'addHotKeys',
    value: function addHotKeys() {
      var actionNameToKeyMap = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var actionNameToHandlersMap = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var options = arguments[2];

      if (this.resetOnNextFocus || this.keyMaps) {
        /**
         * We know components have just lost focus or keymaps have already been built,
         * meaning we are either anticipating a new set of components to be focused or
         * we are receiving notice of a component being focused when we aren't expecting it.
         * In either case, the internal state needs to be reset.
         */
        this._reset();
        this.resetOnNextFocus = false;
      }

      this.componentId = this.componentList.length;

      this._addComponentToList(this.componentId, actionNameToKeyMap, actionNameToHandlersMap, options);

      this.logger.debug(this._logPrefix(this.componentId, { eventId: false }) + ' Focused. \n');

      this.logger.verbose(this._logPrefix(this.componentId, { eventId: false }) + ' Component options:\n', (0, _printComponent2.default)(this.componentList[this.componentId]));

      return [this.focusTreeId, this.componentId];
    }

    /**
     * Handles when a HotKeys component that is in focus updates its props and changes
     * either the keyMap or handlers prop value
     * @param {FocusTreeId} focusTreeId - The ID of the focus tree the component is part of.
     *        Used to identify (and ignore) stale updates.
     * @param {ComponentId} componentId - The component index of the component to
     *        update
     * @param {KeyMap} actionNameToKeyMap - Map of key sequences to action names
     * @param {HandlersMap} actionNameToHandlersMap - Map of action names to handler
     *        functions
     * @param {Object} options Hash of options that configure how the actions
     *        and handlers are associated and called.
     */

  }, {
    key: 'updateHotKeys',
    value: function updateHotKeys(focusTreeId, componentId) {
      var actionNameToKeyMap = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var actionNameToHandlersMap = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
      var options = arguments[4];

      if (focusTreeId !== this.focusTreeId || !this.componentList[componentId]) {
        return;
      }

      this.componentList[componentId] = this._buildComponentOptions(componentId, actionNameToKeyMap, actionNameToHandlersMap, options);

      this.logger.debug(this._logPrefix(componentId, { focusTreeId: focusTreeId, eventId: false }) + ' Received new props.');

      this.logger.verbose(this._logPrefix(componentId, { focusTreeId: focusTreeId, eventId: false }) + ' Component options:\n', (0, _printComponent2.default)(this.componentList[componentId]));
    }

    /**
     * Handles when a component loses focus by resetting the internal state, ready to
     * receive the next tree of focused HotKeys components
     * @param {FocusTreeId} focusTreeId - Id of focus tree component thinks it's
     *        apart of
     * @param {ComponentId} componentId - Index of component that is blurring
     * @returns {Boolean} Whether the component still has event propagation yet to handle
     */

  }, {
    key: 'removeHotKeys',
    value: function removeHotKeys(focusTreeId, componentId) {
      if (!this.resetOnNextFocus) {
        this.resetOnNextFocus = true;
      }

      var outstandingEventPropagation = this.eventPropagationState.previousComponentIndex + 1 < componentId;

      this.logger.debug('' + this._logPrefix(componentId, { focusTreeId: focusTreeId, eventId: false }), 'Lost focus' + (outstandingEventPropagation ? ' (Key event has yet to propagate through it)' : '') + '.');

      return outstandingEventPropagation;
    }

    /********************************************************************************
     * Recording key events
     ********************************************************************************/

    /**
     * Records a keydown keyboard event and matches it against the list of pre-registered
     * event handlers, calling the first matching handler with the highest priority if
     * one exists.
     *
     * This method is called many times as a keyboard event bubbles up through the React
     * render tree. The event is only registered the first time it is seen and results
     * of some calculations are cached. The event is matched against the handlers registered
     * at each component level, to ensure the proper handler declaration scoping.
     * @param {KeyboardEvent} event - Event containing the key name and state
     * @param {FocusTreeId} focusTreeId - Id of focus tree component thinks it's apart of
     * @param {ComponentId} componentId - The id of the component that is currently handling
     *        the keyboard event as it bubbles towards the document root.
     * @param {Object} options - Hash of options that configure how the event is handled.
     * @returns Whether the event was discarded because it was part of an old focus tree
     */

  }, {
    key: 'handleKeydown',
    value: function handleKeydown(event, focusTreeId, componentId) {
      var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

      var _key = (0, _normalizeKeyName2.default)(event.key);

      if (focusTreeId !== this.focusTreeId) {
        this.logger.debug(this._logPrefix(componentId) + '\' Ignored \'' + _key + '\' keydown event because it had an old focus tree id: ' + focusTreeId + '.');

        this._ignoreEvent(event, componentId);

        return true;
      }

      if (this._alreadyEstablishedShouldIgnoreEvent()) {
        this.logger.debug(this._logPrefix(componentId) + ' Ignored \'' + _key + '\' keydown event because ignoreEventsFilter rejected it.');

        this._ignoreEvent(event, componentId);

        return false;
      }

      if (this._isNewKeyEvent(componentId)) {
        this._setNewEventParameters(event, _KeyEventBitmapIndex2.default.keydown);

        /**
         * We know that this is a new key event and not the same event bubbling up
         * the React render tree towards the document root, so perform actions specific
         * to the first time an event is seen
         */

        this._setIgnoreEventFlag(event, options);

        if (this._alreadyEstablishedShouldIgnoreEvent()) {
          this.logger.debug(this._logPrefix(componentId) + ' Ignored \'' + _key + '\' keydown event because ignoreEventsFilter rejected it.');

          this._ignoreEvent(event, componentId);

          return false;
        }

        this.logger.debug(this._logPrefix(componentId) + ' New \'' + _key + '\' keydown event.');

        this._checkForModifierFlagDiscrepancies(event);

        var keyInCurrentCombination = !!this._getCurrentKeyState(_key);

        if (keyInCurrentCombination || this.keyCombinationIncludesKeyUp) {
          this._startNewKeyCombination(_key, _KeyEventBitmapIndex2.default.keydown, focusTreeId, componentId);
        } else {
          this._addToCurrentKeyCombination(_key, _KeyEventBitmapIndex2.default.keydown, focusTreeId, componentId);
        }
      }

      this._callHandlerIfActionNotHandled(event, _key, _KeyEventBitmapIndex2.default.keydown, componentId, focusTreeId);

      if (!(0, _hasKeyPressEvent2.default)(_key) && _Configuration2.default.option('simulateMissingKeyPressEvents')) {
        /**
         * If a key does not have a keypress event, we save the details of the keydown
         * event to simulate the keypress event, as the keydown event bubbles through
         * the last focus-only HotKeysComponent
         */
        event.persist();

        this.keypressEventsToSimulate.push({
          event: event, focusTreeId: focusTreeId, componentId: componentId, options: options, key: _key
        });
      }

      if (this._isFocusTreeRoot(componentId)) {
        if (!this.keyEventManager.isGlobalListenersBound()) {
          this.simulatePendingKeyPressEvents(_key);
        }
        /**
         * else, we wait for keydown event to propagate through global strategy
         * before we simulate the keypress
         */
      }

      this._updateEventPropagationHistory(componentId);

      return false;
    }
  }, {
    key: 'simulatePendingKeyPressEvents',
    value: function simulatePendingKeyPressEvents() {
      var _this2 = this;

      if (this.keypressEventsToSimulate.length > 0) {
        _KeyEventCounter2.default.incrementId();
      }

      this.keypressEventsToSimulate.forEach(function (_ref) {
        var event = _ref.event,
            focusTreeId = _ref.focusTreeId,
            componentId = _ref.componentId,
            options = _ref.options;

        _this2.handleKeypress(event, focusTreeId, componentId, options);
      });

      this.keypressEventsToSimulate = [];

      /**
       * If an event gets handled and causes a focus shift, then subsequent components
       * will ignore the event (including the root component) and the conditions to
       * reset the propagation state are never met - so we ensure that after we are done
       * simulating the keypress event, the propagation state is reset for the following
       * keyup
       */
      this._clearEventPropagationState();
    }

    /**
     * Records a keypress keyboard event and matches it against the list of pre-registered
     * event handlers, calling the first matching handler with the highest priority if
     * one exists.
     *
     * This method is called many times as a keyboard event bubbles up through the React
     * render tree. The event is only registered the first time it is seen and results
     * of some calculations are cached. The event is matched against the handlers registered
     * at each component level, to ensure the proper handler declaration scoping.
     * @param {KeyboardEvent} event - Event containing the key name and state
     * @param {FocusTreeId} focusTreeId Id - of focus tree component thinks it's apart of
     * @param {ComponentId} componentId - The index of the component that is currently handling
     *        the keyboard event as it bubbles towards the document root.
     * @param {Object} options - Hash of options that configure how the event
     *        is handled.
     */

  }, {
    key: 'handleKeypress',
    value: function handleKeypress(event, focusTreeId, componentId, options) {
      var _key = (0, _normalizeKeyName2.default)(event.key);

      if (focusTreeId !== this.focusTreeId) {
        this.logger.debug(this._logPrefix(componentId) + ' Ignored \'' + _key + '\' keypress event because it had an old focus tree id: ' + focusTreeId + '.');

        this._ignoreEvent(event, componentId);

        return true;
      }

      if (this._alreadyEstablishedShouldIgnoreEvent()) {
        this.logger.debug(this._logPrefix(componentId) + ' Ignored \'' + _key + '\' keypress event because ignoreEventsFilter rejected it.');

        this._ignoreEvent(event, componentId);

        return;
      }

      if (this._isNewKeyEvent(componentId)) {
        this._setNewEventParameters(event, _KeyEventBitmapIndex2.default.keypress);

        /**
         * We know that this is a new key event and not the same event bubbling up
         * the React render tree towards the document root, so perform actions specific
         * to the first time an event is seen
         */

        this._setIgnoreEventFlag(event, options);

        if (this._alreadyEstablishedShouldIgnoreEvent()) {
          this.logger.debug(this._logPrefix(componentId) + ' Ignored \'' + _key + '\' keypress event because ignoreEventsFilter rejected it.');

          this._ignoreEvent(event, componentId);

          return;
        }

        this.logger.debug(this._logPrefix(componentId) + ' New \'' + _key + '\' keypress event.');

        /**
         * Add new key event to key combination history
         */

        var keyCombination = this._getCurrentKeyState(_key);
        var alreadySeenKeyInCurrentCombo = keyCombination && (keyCombination[_KeyEventSequenceIndex2.default.current][_KeyEventBitmapIndex2.default.keypress] || keyCombination[_KeyEventSequenceIndex2.default.current][_KeyEventBitmapIndex2.default.keyup]);

        if (alreadySeenKeyInCurrentCombo) {
          this._startNewKeyCombination(_key, _KeyEventBitmapIndex2.default.keypres, focusTreeId, componentId);
        } else {
          this._addToCurrentKeyCombination(_key, _KeyEventBitmapIndex2.default.keypress, focusTreeId, componentId);
        }
      }

      this._callHandlerIfActionNotHandled(event, _key, _KeyEventBitmapIndex2.default.keypress, componentId, focusTreeId);

      this._updateEventPropagationHistory(componentId);
    }

    /**
     * Records a keyup keyboard event and matches it against the list of pre-registered
     * event handlers, calling the first matching handler with the highest priority if
     * one exists.
     *
     * This method is called many times as a keyboard event bubbles up through the React
     * render tree. The event is only registered the first time it is seen and results
     * of some calculations are cached. The event is matched against the handlers registered
     * at each component level, to ensure the proper handler declaration scoping.
     * @param {KeyboardEvent} event Event containing the key name and state
     * @param {FocusTreeId} focusTreeId Id of focus tree component thinks it's apart of
     * @param {ComponentId} componentId The index of the component that is currently handling
     *        the keyboard event as it bubbles towards the document root.
     * @param {Object} options Hash of options that configure how the event
     *        is handled.
     */

  }, {
    key: 'handleKeyup',
    value: function handleKeyup(event, focusTreeId, componentId, options) {
      var _key = (0, _normalizeKeyName2.default)(event.key);

      if (focusTreeId !== this.focusTreeId) {
        this.logger.debug(this._logPrefix(componentId) + ' Ignored \'' + _key + '\' keyup event because it had an old focus tree id: ' + focusTreeId + '.');

        this._ignoreEvent(event, componentId);

        return true;
      }

      if (this._alreadyEstablishedShouldIgnoreEvent()) {
        this.logger.debug(this._logPrefix(componentId) + ' Ignored \'' + _key + '\' keyup event because ignoreEventsFilter rejected it.');

        this._ignoreEvent(event, componentId);

        return;
      }

      if (this._isNewKeyEvent(componentId)) {
        this._setNewEventParameters(event, _KeyEventBitmapIndex2.default.keyup);

        /**
         * We know that this is a new key event and not the same event bubbling up
         * the React render tree towards the document root, so perform actions specific
         * to the first time an event is seen
         */

        this._setIgnoreEventFlag(event, options);

        if (this._alreadyEstablishedShouldIgnoreEvent()) {
          this.logger.debug(this._logPrefix(componentId) + ' Ignored \'' + _key + '\' keyup event because ignoreEventsFilter rejected it.');

          this._ignoreEvent(event, componentId);

          return;
        }

        this.logger.debug(this._logPrefix(componentId) + ' New \'' + _key + '\' keyup event.');

        var keyCombination = this._getCurrentKeyState(_key);

        var alreadySeenKeyEventInCombo = keyCombination && keyCombination[_KeyEventSequenceIndex2.default.current][_KeyEventBitmapIndex2.default.keyup];

        if (alreadySeenKeyEventInCombo) {
          this._startNewKeyCombination(_key, _KeyEventBitmapIndex2.default.keyup, focusTreeId, componentId);
        } else {
          this._addToCurrentKeyCombination(_key, _KeyEventBitmapIndex2.default.keyup, focusTreeId, componentId);
        }
      }

      this._callHandlerIfActionNotHandled(event, _key, _KeyEventBitmapIndex2.default.keyup, componentId, focusTreeId);

      this._updateEventPropagationHistory(componentId);
    }
  }, {
    key: '_ignoreEvent',
    value: function _ignoreEvent(event, componentId) {
      if (this._stopEventPropagationAfterIgnoringIfEnabled(event, componentId)) {
        this._updateEventPropagationHistory(componentId, { forceReset: true });
      } else {
        this._updateEventPropagationHistory(componentId);
      }
    }

    /**
     * Whether KeyEventManager should ignore the event that is currently being handled
     * @returns {Boolean} Whether to ignore the event
     *
     * Do not override this method. Use setIgnoreEventsCondition() instead.
     * @private
     */

  }, {
    key: '_alreadyEstablishedShouldIgnoreEvent',
    value: function _alreadyEstablishedShouldIgnoreEvent() {
      return this.eventPropagationState.ignoreEvent;
    }

    /**
     * Returns whether this is a previously seen event bubbling up to render tree towards
     * the document root, or whether it is a new event that has not previously been seen.
     * @param {ComponentId} componentId Index of the component currently handling
     *        the keyboard event
     * @return {Boolean} If the event has been seen before
     * @private
     */

  }, {
    key: '_isNewKeyEvent',
    value: function _isNewKeyEvent(componentId) {
      var previousComponentIndex = this.eventPropagationState.previousComponentIndex;


      return previousComponentIndex === -1 || previousComponentIndex >= componentId;
    }
  }, {
    key: '_updateEventPropagationHistory',
    value: function _updateEventPropagationHistory(componentId) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { forceReset: false };

      if (options.forceReset || this._isFocusTreeRoot(componentId)) {
        this._clearEventPropagationState();
      } else {
        this.eventPropagationState.previousComponentIndex = componentId;
      }
    }

    /**
     * Sets the ignoreEvent flag so that subsequent handlers of the same event
     * do not have to re-evaluate whether to ignore the event or not as it bubbles
     * up towards the document root
     * @param {KeyboardEvent} event The event to decide whether to ignore
     * @param {Object} options Options containing the function to use
     *        to set the ignoreEvent flag
     * @param {Function} options.ignoreEventsCondition Function used to for setting
     *        the ignoreEvent flag
     * @private
     */

  }, {
    key: '_setIgnoreEventFlag',
    value: function _setIgnoreEventFlag(event, options) {
      this.eventPropagationState.ignoreEvent = options.ignoreEventsCondition(event);
    }
  }, {
    key: 'ignoreEvent',
    value: function ignoreEvent(event) {
      this.eventPropagationState.ignoreEvent = true;
    }
  }, {
    key: '_isFocusTreeRoot',
    value: function _isFocusTreeRoot(componentId) {
      return componentId >= this.componentList.length - 1;
    }
  }, {
    key: '_setNewEventParameters',
    value: function _setNewEventParameters(event, type) {
      _KeyEventCounter2.default.incrementId();

      this.currentEvent = {
        key: event.key,
        type: type,
        handled: false
      };
    }
  }, {
    key: '_startNewKeyCombination',
    value: function _startNewKeyCombination(keyName, eventBitmapIndex, focusTreeId, componentId) {
      _get(FocusOnlyKeyEventStrategy.prototype.__proto__ || Object.getPrototypeOf(FocusOnlyKeyEventStrategy.prototype), '_startNewKeyCombination', this).call(this, keyName, eventBitmapIndex);

      this.logger.verbose(this._logPrefix(componentId, { focusTreeId: focusTreeId }) + ' Started a new combination with \'' + keyName + '\'.');

      this.logger.verbose(this._logPrefix(componentId, { focusTreeId: focusTreeId }) + ' Key history: ' + (0, _printComponent2.default)(this.keyCombinationHistory) + '.');
    }
  }, {
    key: '_addToCurrentKeyCombination',
    value: function _addToCurrentKeyCombination(keyName, eventBitmapIndex, focusTreeId, componentId) {
      _get(FocusOnlyKeyEventStrategy.prototype.__proto__ || Object.getPrototypeOf(FocusOnlyKeyEventStrategy.prototype), '_addToCurrentKeyCombination', this).call(this, keyName, eventBitmapIndex);

      if (eventBitmapIndex === _KeyEventBitmapIndex2.default.keydown) {
        this.logger.verbose(this._logPrefix(componentId, { focusTreeId: focusTreeId }) + ' Added \'' + keyName + '\' to current combination: \'' + this._getCurrentKeyCombination().ids[0] + '\'.');
      }

      this.logger.verbose(this._logPrefix(componentId, { focusTreeId: focusTreeId }) + ' Key history: ' + (0, _printComponent2.default)(this.keyCombinationHistory) + '.');
    }

    /********************************************************************************
     * Matching and calling handlers
     ********************************************************************************/

    /**
     * Calls the first handler that matches the current key event if the action has not
     * already been handled in a more deeply nested component
     * @param {KeyboardEvent} event Keyboard event object to be passed to the handler
     * @param {NormalizedKeyName} keyName Normalized key name
     * @param {KeyEventBitmapIndex} eventBitmapIndex The bitmap index of the current key event type
     * @param {FocusTreeId} focusTreeId Id of focus tree component thinks it's apart of
     * @param {ComponentId} componentId Index of the component that is currently handling
     *        the keyboard event
     * @private
     */

  }, {
    key: '_callHandlerIfActionNotHandled',
    value: function _callHandlerIfActionNotHandled(event, keyName, eventBitmapIndex, componentId, focusTreeId) {
      var eventName = (0, _describeKeyEvent2.default)(eventBitmapIndex);
      var combinationName = this._describeCurrentKeyCombination();

      if (this.keyMapEventBitmap[eventBitmapIndex]) {
        if (this.eventPropagationState.actionHandled) {
          this.logger.debug(this._logPrefix(componentId, { focusTreeId: focusTreeId }) + ' Ignored \'' + combinationName + '\' ' + eventName + ' as it has already been handled.');
        } else {
          this.logger.verbose(this._logPrefix(componentId, { focusTreeId: focusTreeId }) + ' Attempting to find action matching \'' + combinationName + '\' ' + eventName + ' . . .');

          var previousComponentIndex = this.eventPropagationState.previousComponentIndex;


          var handlerWasCalled = this._callMatchingHandlerClosestToEventTarget(event, keyName, eventBitmapIndex, componentId, previousComponentIndex === -1 ? 0 : previousComponentIndex);

          if (handlerWasCalled) {
            this.eventPropagationState.actionHandled = true;
            this.currentEvent.handled = true;
          }
        }
      } else {
        this.logger.verbose(this._logPrefix(componentId, { focusTreeId: focusTreeId }) + ' Ignored \'' + combinationName + '\' ' + eventName + ' because it doesn\'t have any ' + eventName + ' handlers.');
      }
    }

    /********************************************************************************
     * Logging
     ********************************************************************************/

  }, {
    key: '_logPrefix',
    value: function _logPrefix(componentId) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var logIcons = _Logger2.default.logIcons;
      var eventIcons = _Logger2.default.eventIcons;
      var componentIcons = _Logger2.default.componentIcons;

      var base = 'HotKeys (';

      if (options.focusTreeId !== false) {
        var focusTreeId = (0, _isUndefined2.default)(options.focusTreeId) ? this.focusTreeId : options.focusTreeId;
        base += 'FT' + focusTreeId + logIcons[focusTreeId % logIcons.length] + '-';
      }

      if (options.eventId !== false) {
        var eventId = (0, _isUndefined2.default)(options.eventId) ? _KeyEventCounter2.default.getId() : options.eventId;

        base += 'E' + eventId + eventIcons[eventId % eventIcons.length] + '-';
      }

      return base + ('C' + componentId + componentIcons[componentId % componentIcons.length] + '):');
    }
  }]);

  return FocusOnlyKeyEventStrategy;
}(_AbstractKeyEventStrategy2.default);

exports.default = FocusOnlyKeyEventStrategy;