var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import KeyEventBitmapManager from '../KeyEventBitmapManager';
import KeyEventBitmapIndex from '../../const/KeyEventBitmapIndex';
import KeyEventSequenceIndex from '../../const/KeyEventSequenceIndex';
import AbstractKeyEventStrategy from './AbstractKeyEventStrategy';
import capitalize from '../../utils/string/capitalize';
import hasKeyPressEvent from '../../helpers/resolving-handlers/hasKeyPressEvent';
import describeKeyEvent from '../../helpers/logging/describeKeyEvent';
import KeyEventCounter from '../KeyEventCounter';
import Logger from '../Logger';
import removeAtIndex from '../../utils/array/removeAtIndex';
import isUndefined from '../../utils/isUndefined';
import getEventKey from '../../vendor/react-dom/getEventKey';
import printComponent from '../../helpers/logging/printComponent';
import normalizeKeyName from '../../helpers/resolving-handlers/normalizeKeyName';
import Configuration from '../Configuration';

/**
 * Defines behaviour for dealing with key maps defined in global HotKey components
 * @class
 */

var GlobalKeyEventStrategy = function (_AbstractKeyEventStra) {
  _inherits(GlobalKeyEventStrategy, _AbstractKeyEventStra);

  /********************************************************************************
   * Init & Reset
   ********************************************************************************/

  function GlobalKeyEventStrategy() {
    var configuration = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var keyEventManager = arguments[1];

    _classCallCheck(this, GlobalKeyEventStrategy);

    /**
     * Set state that doesn't get cleared each time a new new component is mounted
     * or unmounted
     * @type {number}
     */

    /**
     * Counter to keep track of what component ID should be allocated next
     * @type {ComponentId}
     */
    var _this = _possibleConstructorReturn(this, (GlobalKeyEventStrategy.__proto__ || Object.getPrototypeOf(GlobalKeyEventStrategy)).call(this, configuration, keyEventManager));
    /**
     * Set state that gets cleared every time a component gets mounted or unmounted
     */


    _this.componentId = -1;

    /**
     * Whether the global key event handlers have been bound to document yet or not
     * @type {boolean}
     */
    _this.listenersBound = false;

    _this.eventOptions = {
      ignoreEventsCondition: Configuration.option('ignoreEventsCondition')
    };
    return _this;
  }

  _createClass(GlobalKeyEventStrategy, [{
    key: '_reset',
    value: function _reset() {
      _get(GlobalKeyEventStrategy.prototype.__proto__ || Object.getPrototypeOf(GlobalKeyEventStrategy.prototype), '_reset', this).call(this);

      /**
       * Set of ComponentOptions indexed by ComponentId to allow efficient retrieval
       * when components need to be updated or unmounted by their ComponentId
       * @type {Object<ComponentId, ComponentOptions>}
       */
      this.componentIdDict = {};
    }

    /********************************************************************************
     * Registering key maps and handlers
     ********************************************************************************/

    /**
     * Registers the actions and handlers of a HotKeys component that has mounted
     * @param {KeyMap} actionNameToKeyMap - Map of actions to key expressions
     * @param {HandlersMap} actionNameToHandlersMap - Map of actions to handler functions
     * @param {Object} options Hash of options that configure how the actions
     *        and handlers are associated and called.
     * @param {Object} eventOptions - Options for how the event should be handled
     * @returns {ComponentId} A unique component ID to assign to the focused HotKeys
     *        component and passed back when handling a key event
     */

  }, {
    key: 'addHotKeys',
    value: function addHotKeys() {
      var actionNameToKeyMap = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var actionNameToHandlersMap = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var options = arguments[2];
      var eventOptions = arguments[3];

      this.eventOptions = eventOptions;

      this.componentId += 1;

      this._addComponentToList(this.componentId, actionNameToKeyMap, actionNameToHandlersMap, options);

      this._updateDocumentHandlers();

      this.logger.debug(this._logPrefix(this.componentId, { eventId: false }), 'Mounted.');

      this.logger.verbose(this._logPrefix(this.componentId, { eventId: false }), 'Component options: \n', printComponent(this._getComponent(this.componentId)));

      return this.componentId;
    }

    /**
     * Handles when a mounted global HotKeys component updates its props and changes
     * either the keyMap or handlers prop value
     * @param {ComponentId} componentId - The component index of the component to
     *        update
     * @param {KeyMap} actionNameToKeyMap - Map of actions to key expressions
     * @param {HandlersMap} actionNameToHandlersMap - Map of actions to handler functions
     * @param {Object} options Hash of options that configure how the actions
     *        and handlers are associated and called.
     * @param {Object} eventOptions - Options for how the event should be handled
     */

  }, {
    key: 'updateHotKeys',
    value: function updateHotKeys(componentId) {
      var actionNameToKeyMap = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var actionNameToHandlersMap = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var options = arguments[3];
      var eventOptions = arguments[4];

      this.eventOptions = eventOptions;

      var componentPosition = this._getComponentPosition(componentId);

      /**
       * Manually update the registered key map state, usually reset using
       * _resetRegisteredKeyMapsState() method
       */

      this.componentList[componentPosition] = this._buildComponentOptions(componentId, actionNameToKeyMap, actionNameToHandlersMap, options);

      this._updateLongestKeySequenceIfNecessary(componentId);

      /**
       * Reset strategy state specific to the global strategy
       */

      this._updateDocumentHandlers();

      /**
       * Reset handler resolution state
       */
      this._resetHandlerResolutionState();

      this.logger.debug(this._logPrefix(this.componentId, { eventId: false }), 'Global component ' + componentId + ' updated.');

      this.logger.verbose(this._logPrefix(this.componentId, { eventId: false }), 'Component options: \n', printComponent(this._getComponent(componentId)));
    }

    /**
     * Handles when a component is unmounted
     * @param {ComponentId} componentId - Index of component that is being unmounted
     */

  }, {
    key: 'removeHotKeys',
    value: function removeHotKeys(componentId) {
      var _getComponentAndPosit = this._getComponentAndPosition(componentId),
          _getComponentAndPosit2 = _slicedToArray(_getComponentAndPosit, 2),
          keyMapEventBitmap = _getComponentAndPosit2[0].keyMapEventBitmap,
          componentPosition = _getComponentAndPosit2[1];

      /**
       * Manually update the registered key map state, usually reset using
       * _resetRegisteredKeyMapsState() method
       */


      this.componentList = removeAtIndex(this.componentList, componentPosition);

      this._updateLongestKeySequenceIfNecessary(componentId);

      /**
       * Reset strategy state specific to the global strategy
       */
      this._updateComponentIndexDictFromList({ startingAt: componentPosition });

      this._updateDocumentHandlers(keyMapEventBitmap, KeyEventBitmapManager.newBitmap());

      /**
       * Reset handler resolution state
       */
      this._resetHandlerResolutionState();

      this.logger.debug(this._logPrefix(this.componentId, { eventId: false }), 'Unmounted global component ' + componentId);
    }
  }, {
    key: '_addComponentToList',
    value: function _addComponentToList(componentId) {
      var actionNameToKeyMap = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var actionNameToHandlersMap = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var options = arguments[3];

      _get(GlobalKeyEventStrategy.prototype.__proto__ || Object.getPrototypeOf(GlobalKeyEventStrategy.prototype), '_addComponentToList', this).call(this, componentId, actionNameToKeyMap, actionNameToHandlersMap, options);

      this._setComponentPosition(componentId, this.componentList.length - 1);
    }
  }, {
    key: '_setComponentPosition',
    value: function _setComponentPosition(componentId, position) {
      this.componentIdDict[componentId] = position;
    }
  }, {
    key: '_updateLongestKeySequenceIfNecessary',
    value: function _updateLongestKeySequenceIfNecessary(componentId) {
      var _this2 = this;

      if (componentId === this.longestSequenceComponentIndex) {
        this.longestSequence = 1;

        this.componentList.forEach(function (_ref) {
          var longestSequence = _ref.longestSequence;

          if (longestSequence > _this2.longestSequence) {
            _this2.longestSequence = longestSequence;
          }
        });
      }
    }
  }, {
    key: '_updateComponentIndexDictFromList',
    value: function _updateComponentIndexDictFromList() {
      var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : { startingAt: 0 };

      var counter = options.startAt;

      while (counter < this.componentList.length) {
        this._setComponentPosition(this.componentList[counter].componentId, counter);
      }
    }
  }, {
    key: '_updateDocumentHandlers',
    value: function _updateDocumentHandlers() {
      var _this3 = this;

      var listenersShouldBeBound = this.keyMapEventBitmap.some(function (eventType) {
        return eventType;
      });

      if (!this.listenersBound && listenersShouldBeBound) {
        var _loop = function _loop(bitmapIndex) {
          var eventName = describeKeyEvent(bitmapIndex);

          document['on' + eventName] = function (keyEvent) {
            _this3.keyEventManager['handleGlobal' + capitalize(eventName)](keyEvent);
          };

          _this3.logger.debug(_this3._logPrefix(_this3.componentId, { eventId: false }), 'Bound handler handleGlobal' + capitalize(eventName) + '() to document.on' + eventName + '()');
        };

        for (var bitmapIndex = 0; bitmapIndex < this.keyMapEventBitmap.length; bitmapIndex++) {
          _loop(bitmapIndex);
        }

        this.listenersBound = true;
      } else if (this.listenersBound && !listenersShouldBeBound) {

        for (var bitmapIndex = 0; bitmapIndex < this.keyMapEventBitmap.length; bitmapIndex++) {
          var _eventName = describeKeyEvent(bitmapIndex);

          delete document['on' + _eventName];

          this.logger.debug(this._logPrefix(this.componentId, { eventId: false }), 'Removed handler handleGlobal' + capitalize(_eventName) + '() from document.on' + _eventName + '()');
        }

        this.listenersBound = false;
      }
    }

    /********************************************************************************
     * Recording key events
     ********************************************************************************/

    /**
     * Records a keydown keyboard event and matches it against the list of pre-registered
     * event handlers, calling the first matching handler with the highest priority if
     * one exists.
     *
     * This method is called once when a keyboard event bubbles up to document, and checks
     * the keymaps for all of the mounted global HotKey components.
     * @param {KeyboardEvent} event - Event containing the key name and state
     */

  }, {
    key: 'handleKeydown',
    value: function handleKeydown(event) {
      this._checkForModifierFlagDiscrepancies(event);

      var _key = normalizeKeyName(getEventKey(event));

      var keyInCurrentCombination = !!this._getCurrentKeyState(_key);

      if (keyInCurrentCombination || this.keyCombinationIncludesKeyUp) {
        this._startNewKeyCombination(_key, KeyEventBitmapIndex.keydown);
      } else {
        this._addToCurrentKeyCombination(_key, KeyEventBitmapIndex.keydown);
      }

      var reactAppHistoryWithEvent = this.keyEventManager.reactAppHistoryWithEvent(_key, KeyEventBitmapIndex.keydown);

      if (reactAppHistoryWithEvent === 'handled') {
        this.logger.debug(this._logPrefix() + ' Ignored \'' + _key + '\' keydown event because React app has already handled it.');
      } else {
        if (reactAppHistoryWithEvent === 'seen') {
          this.logger.debug(this._logPrefix() + ' Received \'' + _key + '\' keydown event (that has already passed through React app).');
        } else {
          KeyEventCounter.incrementId();

          this.logger.debug(this._logPrefix() + ' New \'' + _key + '\' keydown event (that has NOT passed through React app).');
        }

        if (this.eventOptions.ignoreEventsCondition(event)) {
          this.logger.debug(this._logPrefix() + ' Ignored \'' + _key + '\' keydown event because ignoreEventsFilter rejected it.');

          return;
        }

        this._callHandlerIfExists(event, _key, KeyEventBitmapIndex.keydown);

        if (!hasKeyPressEvent(_key)) {
          this.logger.debug(this._logPrefix() + ' Simulating \'' + _key + '\' keypress event because \'' + _key + '\' doesn\'t natively have one.');

          /**
           * We simulate keypress events in the React app before we do it globally
           */
          this.keyEventManager.simulatePendingKeyPressEvents();

          /**
           * If a key does not have a keypress event, we simulate one immediately after
           * the keydown event, to keep the behaviour consistent across all keys
           */
          this.handleKeypress(event);
        }
      }
    }

    /**
     * Records a keypress keyboard event and matches it against the list of pre-registered
     * event handlers, calling the first matching handler with the highest priority if
     * one exists.
     *
     * This method is called once when a keyboard event bubbles up to document, and checks
     * the keymaps for all of the mounted global HotKey components.
     * @param {KeyboardEvent} event - Event containing the key name and state
     */

  }, {
    key: 'handleKeypress',
    value: function handleKeypress(event) {
      var _key = normalizeKeyName(getEventKey(event));

      /**
       * Add new key event to key combination history
       */

      var keyCombination = this._getCurrentKeyState(_key);

      var alreadySeenKeyInCurrentCombo = keyCombination && (keyCombination[KeyEventSequenceIndex.current][KeyEventBitmapIndex.keypress] || keyCombination[KeyEventSequenceIndex.current][KeyEventBitmapIndex.keyup]);

      if (alreadySeenKeyInCurrentCombo) {
        this._startNewKeyCombination(_key, KeyEventBitmapIndex.keypress);
      } else {
        this._addToCurrentKeyCombination(_key, KeyEventBitmapIndex.keypress);
      }

      var reactAppHistoryWithEvent = this.keyEventManager.reactAppHistoryWithEvent(_key, KeyEventBitmapIndex.keypress);

      if (reactAppHistoryWithEvent === 'handled') {
        this.logger.debug(this._logPrefix() + ' Ignored \'' + _key + '\' keypress event because React app has already handled it.');

        return false;
      } else {
        if (reactAppHistoryWithEvent === 'seen') {
          this.logger.debug(this._logPrefix() + ' \'' + _key + '\' keypress event (that has already passed through React app).');
        } else {
          KeyEventCounter.incrementId();

          this.logger.debug(this._logPrefix() + ' New \'' + _key + '\' keypress event (that has NOT passed through React app).');
        }

        if (this.eventOptions.ignoreEventsCondition(event)) {
          this.logger.debug(this._logPrefix() + ' Ignored \'' + _key + '\' keypress event because ignoreEventsFilter rejected it.');

          return false;
        }

        this._callHandlerIfExists(event, _key, KeyEventBitmapIndex.keypress);
      }
    }

    /**
     * Records a keyup keyboard event and matches it against the list of pre-registered
     * event handlers, calling the first matching handler with the highest priority if
     * one exists.
     *
     * This method is called once when a keyboard event bubbles up to document, and checks
     * the keymaps for all of the mounted global HotKey components.
     * @param {KeyboardEvent} event - Event containing the key name and state
     */

  }, {
    key: 'handleKeyup',
    value: function handleKeyup(event) {
      var _key = normalizeKeyName(getEventKey(event));

      var keyCombination = this._getCurrentKeyState(_key);

      var alreadySeenKeyInCurrentCombo = keyCombination && keyCombination[KeyEventSequenceIndex.current][KeyEventBitmapIndex.keyup];

      if (alreadySeenKeyInCurrentCombo) {
        this._startNewKeyCombination(_key, KeyEventBitmapIndex.keyup);
      } else {
        this._addToCurrentKeyCombination(_key, KeyEventBitmapIndex.keyup);
      }

      var reactAppHistoryWithEvent = this.keyEventManager.reactAppHistoryWithEvent(_key, KeyEventBitmapIndex.keyup);

      if (reactAppHistoryWithEvent === 'handled') {
        this.logger.debug(this._logPrefix() + ' Ignored \'' + _key + '\' keyup event because React app has already handled it.');

        return false;
      } else {
        if (reactAppHistoryWithEvent === 'seen') {
          this.logger.debug(this._logPrefix() + ' \'' + _key + '\' keyup event (that has already passed through React app).');
        } else {
          KeyEventCounter.incrementId();

          this.logger.debug(this._logPrefix() + ' New \'' + _key + '\' keyup event (that has NOT passed through React app).');
        }

        if (this.eventOptions.ignoreEventsCondition(event)) {
          this.logger.debug(this._logPrefix() + ' Ignored \'' + _key + '\' keyup event because ignoreEventsFilter rejected it.');

          return false;
        }

        this._callHandlerIfExists(event, _key, KeyEventBitmapIndex.keyup);
      }
    }
  }, {
    key: '_startNewKeyCombination',
    value: function _startNewKeyCombination(keyName, eventBitmapIndex) {
      _get(GlobalKeyEventStrategy.prototype.__proto__ || Object.getPrototypeOf(GlobalKeyEventStrategy.prototype), '_startNewKeyCombination', this).call(this, keyName, eventBitmapIndex);

      this.logger.verbose(this._logPrefix() + ' Started a new combination with \'' + keyName + '\'.');

      this.logger.verbose(this._logPrefix() + ' Key history: ' + printComponent(this.keyCombinationHistory) + '.');
    }
  }, {
    key: '_addToCurrentKeyCombination',
    value: function _addToCurrentKeyCombination(keyName, eventBitmapIndex) {
      _get(GlobalKeyEventStrategy.prototype.__proto__ || Object.getPrototypeOf(GlobalKeyEventStrategy.prototype), '_addToCurrentKeyCombination', this).call(this, keyName, eventBitmapIndex);

      if (eventBitmapIndex === KeyEventBitmapIndex.keydown) {
        this.logger.verbose(this._logPrefix() + ' Added \'' + keyName + '\' to current combination: \'' + this._getCurrentKeyCombination().ids[0] + '\'.');
      }

      this.logger.verbose(this._logPrefix() + ' Key history: ' + printComponent(this.keyCombinationHistory) + '.');
    }
  }, {
    key: '_getComponentPosition',
    value: function _getComponentPosition(componentId) {
      return this.componentIdDict[componentId];
    }
  }, {
    key: '_getComponent',
    value: function _getComponent(componentId) {
      var componentPosition = this._getComponentPosition(componentId);
      return this.componentList[componentPosition];
    }
  }, {
    key: '_getComponentAndPosition',
    value: function _getComponentAndPosition(componentId) {
      var componentPosition = this._getComponentPosition(componentId);
      return [this.componentList[componentPosition], componentPosition];
    }

    /********************************************************************************
     * Matching and calling handlers
     ********************************************************************************/

  }, {
    key: '_callHandlerIfExists',
    value: function _callHandlerIfExists(event, keyName, eventBitmapIndex) {
      var eventName = describeKeyEvent(eventBitmapIndex);
      var combinationName = this._describeCurrentKeyCombination();

      if (this.keyMapEventBitmap[eventBitmapIndex]) {
        /**
         * If there is at least one handler for the specified key event type (keydown,
         * keypress, keyup), then attempt to find a handler that matches the current
         * key combination
         */
        this.logger.verbose(this._logPrefix() + ' Attempting to find action matching \'' + combinationName + '\' ' + eventName + ' . . .');

        this._callMatchingHandlerClosestToEventTarget(event, keyName, eventBitmapIndex);
      } else {
        /**
         * If there are no handlers registered for the particular key event type
         * (keydown, keypress, keyup) then skip trying to find a matching handler
         * for the current key combination
         */
        this.logger.debug(this._logPrefix() + ' Ignored \'' + combinationName + '\' ' + eventName + ' because it doesn\'t have any ' + eventName + ' handlers.');
      }
    }
  }, {
    key: '_callMatchingHandlerClosestToEventTarget',
    value: function _callMatchingHandlerClosestToEventTarget(event, keyName, eventBitmapIndex) {
      for (var componentId = 0; componentId < this.componentList.length; componentId++) {
        var matchFound = _get(GlobalKeyEventStrategy.prototype.__proto__ || Object.getPrototypeOf(GlobalKeyEventStrategy.prototype), '_callMatchingHandlerClosestToEventTarget', this).call(this, event, keyName, eventBitmapIndex, componentId, 0);

        if (matchFound) {
          this.logger.debug(this._logPrefix() + ' Searching no further, as handler has been found (and called).');

          return;
        }
      }
    }

    /********************************************************************************
     * Logging
     ********************************************************************************/

  }, {
    key: '_logPrefix',
    value: function _logPrefix(componentId) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var eventIcons = Logger.eventIcons;
      var componentIcons = Logger.componentIcons;

      var base = 'HotKeys (GLOBAL';

      if (options.eventId !== false) {
        var eventId = isUndefined(options.eventId) ? KeyEventCounter.getId() : options.eventId;

        base = base + '-E' + eventId + eventIcons[eventId % eventIcons.length];
      }

      if (isUndefined(componentId)) {
        return base + '):';
      } else {
        return base + '-C' + componentId + componentIcons[componentId % componentIcons.length] + '):';
      }
    }
  }]);

  return GlobalKeyEventStrategy;
}(AbstractKeyEventStrategy);

export default GlobalKeyEventStrategy;